#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

#include "nokia5110/nokia5110.h"

#define NO_Q		5
#define CORRECT		'1'
#define WRONG		'0'

int current_question = 0;
char answer[NO_Q] = {'2', '4', '1', '1', '3'};

char line1[NO_Q][16] = {"What's the", "What's the", "When did WWII", "Where was", "Who directed"};
char line2[NO_Q][16] = {"capital of", "formula for", "start?", "Che Guevara", "Back To The"};
char line3[NO_Q][16] = {"Peru?", "water?", "", "killed?", "Future?"};
char line4[NO_Q][16] = {"", "", "", "", ""};
char line5[NO_Q][16] = {"a. Wien", "a. HCl", "a. 1939", "a. Bolivia", "a. Q.Tarantino"};
char line6[NO_Q][16] = {"b. Lima", "b. NaSo4", "b. 1914", "b. Cuba", "b. B.Chungus"};
char line7[NO_Q][16] = {"c. Guarma", "c. CN-", "c. 1938", "c. Argentina", "c. R.Zemeckis"};
char line8[NO_Q][16] = {"d. Ligma", "d. H2O", "d. 1729", "d. Mexico", "d. S.Spielberg"};

void usart_init(unsigned int baud) {
	UBRR0H = (unsigned char)(baud >> 8);
	UBRR0L = (unsigned char)baud;

	UCSR0B = (1 << TXEN0) | (1 << RXEN0);
	UCSR0C = (1 << USBS0) | (3 << UCSZ00);
}

void send_char(unsigned char data) {
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = data;
}

char recv_char() {
	loop_until_bit_is_set(UCSR0A, RXC0);
	return UDR0;
}

void send_line(char *data)
{
	do {
		send_char(*data++);
	} while (*(data - 1) != '\0');
}

void recv_line(char *data)
{
	int cont = 0;
	char last_recv = 'B';
	while (last_recv != '\0') {
		last_recv = recv_char();
		data[cont++] = last_recv;
	}
}

void comm_test() {
	char line[] = "Comm_Test";

	nokia_lcd_clear();
	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string("Sending...", 1);
	nokia_lcd_render();

	_delay_ms(50000);

	send_line(line);

	nokia_lcd_clear();
	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string("Sent!", 1);
	nokia_lcd_render();

	_delay_ms(50000);
}

void send_question() {
	int current_index = current_question % NO_Q;

	send_line(line1[current_index]);
	_delay_ms(100);

	send_line(line2[current_index]);
	_delay_ms(100);
	
	send_line(line3[current_index]);
	_delay_ms(100);
	
	send_line(line4[current_index]);
	_delay_ms(100);
	
	send_line(line5[current_index]);
	_delay_ms(100);
	
	send_line(line6[current_index]);
	_delay_ms(100);
	
	send_line(line7[current_index]);
	_delay_ms(100);
	
	send_line(line8[current_index]);
	_delay_ms(100);
}

void init_msg() {
	nokia_lcd_clear();

	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string("Welcome to", 1);
	nokia_lcd_set_cursor(0, 12);
	nokia_lcd_write_string("Quizador!", 1);
	nokia_lcd_set_cursor(0, 24);
	nokia_lcd_write_string("Master-Board", 1);
	
	nokia_lcd_render();

	_delay_ms(50000);
}

void init_board() {
	nokia_lcd_init();
	init_msg();
	usart_init(103);	
}


void check_answers() {
	nokia_lcd_clear();

	char slave_response = recv_char();
	char master_response;

	if (slave_response == answer[current_question])
		master_response = CORRECT;
	else
		master_response = WRONG;

	send_char(master_response);
		
	nokia_lcd_set_cursor(0, 0);

	if (slave_response == answer[current_question])
		nokia_lcd_write_string("Correct!", 1);
	else
		nokia_lcd_write_string("Wrong!", 1);

	nokia_lcd_render();
	current_question++;
	_delay_ms(50000);
}

int main() {
	init_board();
	comm_test();

	while (1) {
		send_question();
	
		check_answers();
	}
	return 0;
}
