#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

#include "nokia5110/nokia5110.h"

#define BUTTON_DDR	DDRB
#define BUTTON_PORT	PORTB
#define BUTTON_PIN	PINB

#define BTN_A		PB0
#define BTN_A_PRESSED	(((BUTTON_PIN & (1 << BTN_A)) == 0))

#define BTN_B		PB1
#define BTN_B_PRESSED	(((BUTTON_PIN & (1 << BTN_B)) == 0))

#define BTN_C		PB2
#define BTN_C_PRESSED	(((BUTTON_PIN & (1 << BTN_C)) == 0))

#define BTN_D		PB3
#define BTN_D_PRESSED	(((BUTTON_PIN & (1 << BTN_D)) == 0))

#define BTN_SWITCH		PB4
#define BTN_SWITCH_PRESSED	(((BUTTON_PIN & (1 << BTN_SWITCH)) == 0))

char display_mode;

char line1[16];
char line2[16];
char line3[16];
char line4[16];
char line5[16];
char line6[16];
char line7[16];
char line8[16];

void usart_init(unsigned int baud) {
	UBRR0H = (unsigned char)(baud >> 8);
	UBRR0L = (unsigned char)baud;

	UCSR0B = (1 << TXEN0) | (1 << RXEN0);
	UCSR0C = (1 << USBS0) | (3 << UCSZ00);
}

void send_char(unsigned char data) {
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = data;
}

char recv_char() {
	loop_until_bit_is_set(UCSR0A, RXC0);
	return UDR0;
}

void send_line(char *data) {
	do {
		send_char(*data++);
	} while (*data != '\0');
}

void recv_line(char *data) {
	int cont = 0;
	char last_recv = 'B';
	while (last_recv != '\0') {
		last_recv = recv_char();
		data[cont++] = last_recv;
	}
}

void recv_question() {
	recv_line(line1);
	recv_line(line2);
	recv_line(line3);
	recv_line(line4);
	recv_line(line5);
	recv_line(line6);
	recv_line(line7);
	recv_line(line8);
}

void init_msg() {
	nokia_lcd_clear();

	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string("Welcome to", 1);
	nokia_lcd_set_cursor(0, 12);
	nokia_lcd_write_string("Quizador!", 1);
	nokia_lcd_set_cursor(0, 24);
	nokia_lcd_write_string("Player-Board", 1);
	nokia_lcd_render();

	_delay_ms(50000);
}

void print_question() {
	nokia_lcd_clear();

	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string(line1, 1);
				
	nokia_lcd_set_cursor(0, 12);
	nokia_lcd_write_string(line2, 1);
				
	nokia_lcd_set_cursor(0, 24);
	nokia_lcd_write_string(line3, 1);
				
	nokia_lcd_set_cursor(0, 36);
	nokia_lcd_write_string(line4, 1);
	
	nokia_lcd_render();
	
	_delay_ms(10000);
}

void print_answers() {
	nokia_lcd_clear();

	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string(line5, 1);
			
	nokia_lcd_set_cursor(0, 12);
	nokia_lcd_write_string(line6, 1);
				
	nokia_lcd_set_cursor(0, 24);
	nokia_lcd_write_string(line7, 1);
				
	nokia_lcd_set_cursor(0, 36);
	nokia_lcd_write_string(line8, 1);

	nokia_lcd_render();

	_delay_ms(10000);
}

void check_answer(int index, int a, int b, int c, int d) {
	nokia_lcd_clear();
	nokia_lcd_set_cursor(0, 0);

	char msg;
	if (a)
		msg = '1';
	if (b)
		msg = '2';
	if (c)
		msg = '3';
	if (d)
		msg = '4';

	send_char(msg);
	char master_response = recv_char();
	
	if (master_response == '1')
		nokia_lcd_write_string("Correct!", 1);
	else
		nokia_lcd_write_string("Wrong!", 1);

	nokia_lcd_render();
	
	_delay_ms(50000);
}

void init_board() {
	nokia_lcd_init();
	init_msg();
	
	DDRB &= ~(1 << PB0);
	PORTB |= (1 << PB0);
	DDRB &= ~(1 << PB1);
	PORTB |= (1 << PB1);
	DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);
	DDRB &= ~(1 << PB3);
	PORTB |= (1 << PB3);
	DDRB &= ~(1 << PB4);
	PORTB |= (1 << PB4);

	usart_init(103);
}

void test_comm() {
	char B[50];
	recv_line(B);
	
	nokia_lcd_clear();
	nokia_lcd_set_cursor(0, 0);
	nokia_lcd_write_string(B, 1);
	
	nokia_lcd_render();

	_delay_ms(50000);
}

void start_game() {
	while (1) {
		display_mode = 0;
		recv_question();

		while (1) {
			if (BTN_SWITCH_PRESSED) {
				display_mode++;
			}

			if (display_mode % 2 == 0) {
				print_question();
			} else {
				print_answers();
			}
					
			if (BTN_A_PRESSED || BTN_B_PRESSED || BTN_C_PRESSED || BTN_D_PRESSED) {
				check_answer(0, BTN_A_PRESSED, BTN_B_PRESSED, BTN_C_PRESSED, BTN_D_PRESSED);
				break;
			}
		}
	}
}

int main() {
	init_board();
	test_comm();	
	start_game();
	return 0;
}
